package EntityModel::Async;
BEGIN {
  $EntityModel::Async::VERSION = '0.013';
}
use EntityModel::Class {
};

=head1 NAME

EntityModel::Async - asynchronous helper functions for L<EntityModel>

=head1 VERSION

version 0.013

=head1 SYNOPSIS

 use EntityModel;

=head1 DESCRIPTION


=head1 METHODS

=cut

use EntityModel::Gather;

1;

__END__

=head1 AUTHOR

Tom Molesworth <cpan@entitymodel.com>

=head1 LICENSE

Copyright Tom Molesworth 2008-2011. Licensed under the same terms as Perl itself.
