package EntityModel::Query::GroupField;
BEGIN {
  $EntityModel::Query::GroupField::VERSION = '0.013';
}
use EntityModel::Class {
	_isa => [qw{EntityModel::Query::Field}]
};

=head1 NAME

EntityModel::Query::GroupField - GROUP BY fields

=head1 VERSION

version 0.013

=head1 SYNOPSIS

See L<Entitymodel::Query>.

=head1 DESCRIPTION

See L<Entitymodel::Query>.

=cut

=head1 METHODS

=cut

1;

__END__

=head1 AUTHOR

Tom Molesworth <cpan@entitymodel.com>

=head1 LICENSE

Copyright Tom Molesworth 2008-2011. Licensed under the same terms as Perl itself.
